import asyncio
import functools

import aiorequests

from ..protocol import Protocol


__all__ = (
    "AioRequestsTelegramApi",
)


class AioRequestsTelegramApi:

    def __init__(self, token, delay=1, proxy=None, loop=None, lock=None):
        self.session = aiorequests.Session()
        if proxy is not None:
            self.session.proxies = dict(http=proxy, https=proxy)
        self.proto = Protocol(token)
        self.delay = delay
        self.loop = loop or asyncio.get_event_loop()
        self.lock = lock or asyncio.Lock(loop=self.loop)
        self.last_request_time = 0

    @property
    def token(self):
        return self.proto.token

    def __getattr__(self, name):
        method = getattr(self.proto, name)

        @functools.wraps(method)
        def wrapper(*args, **kwargs):
            return self._run(method(*args, **kwargs))

        return wrapper

    async def _run(self, generator):
        with (await self.lock):
            response = None
            while True:
                request = generator.send(response)
                if request is None:
                    break

                now = self.loop.time()
                timeout = max(0, self.delay - (now - self.last_request_time))
                await asyncio.sleep(timeout, loop=self.loop)

                self.last_request_time = self.loop.time()
                response = await self.session.request(**request._asdict())
                response = response.json()

        return response
